<?php

namespace src\classes;

use src\classes\UserSettings;
use src\classes\Landingpage\RiddleLandingpageLoader;

class RiddlePlugin
{

    /**
     * Starts the plugin and executes all necessary actions/filters/...
     */
    public function run() 
    {
        $this->grabOldConfig(); // reading the old config!

        \add_action('admin_menu', 'loadRiddleMenu'); // add a admin sidebar menu
        \register_activation_hook(__FILE__,  'activatePlugin'); // Add Activate/Deactivate-Hook
        \register_activation_hook(__FILE__,  'deactivatePlugin');

        global $riddleLoader;

        if ($riddleLoader != null && !$this->_isOnRiddleAdminPage()) {
            $riddleLoader->addShortcodeFilter();
            $this->addAction('enqueue_block_editor_assets', array($riddleLoader, 'addRiddleBlock'));
            RiddleLandingpageLoader::addShortcodeFilter();
        }

        if ($this->_isOnRiddleAdminPage()) { // these scripts are only needed on riddle admin pages
            $this->addAction('init', 'src\classes\RiddlePlugin::initSession');
            $this->addAction('wp_loaded', 'src\controller\RiddleAdminController::riddleProcessSettings');
            $this->addAction('wp_loaded', 'src\controller\RiddleCRPController::riddleHandleLandingpageOperation');
            $this->addAction('wp_loaded', 'src\controller\RiddleCRPController::riddleHandleLandingpageDeletion');

            $this->addAction('admin_enqueue_scripts', array($this, 'loadAssets')); // CSS / JS files
        }
    }

    /**
     * Adds an action to wordpress
     */
    public function addAction($action, $callable) 
    {
        add_action($action, $callable);
    }

    /**
     * Adds a stylesheet to wordpress
     * @param $stylesheet (string) stylesheet name WITHOUT file extension
     */
    public static function addStylesheet($stylesheet, bool $local = true) 
    {
        $sheetPath = $local ? RIDDLE_URL_PATH . '/public/css/' . $stylesheet . '.css' : $stylesheet;
        wp_enqueue_style( RIDDLE_PLUGIN_NAME . '_' . $stylesheet, $sheetPath, [], RIDDLE_PLUGIN_VERSION );
    }

    public function addScript( $script, $enqueue = true, $dependencies = []) {
        $scriptPath = RIDDLE_URL_PATH . '/public/js/' . $script . '.js';
        $scriptName = RIDDLE_PLUGIN_NAME . '_' . $script;

        if($enqueue) {
            wp_enqueue_script($scriptName, $scriptPath, $dependencies, RIDDLE_PLUGIN_VERSION);
        } else {
            wp_register_script($scriptName, $scriptPath, $dependencies, RIDDLE_PLUGIN_VERSION);
        }

        return $scriptName;
    }

    /**
     * Loading all the scripts the plugin needs.
     */
    public function addScripts()
    {
        wp_enqueue_media(); // Wordpress media picker
        $this->addScript('plugin');
        $this->addScript('bootstrap');
        $this->addScript('simple-tags');
    }

    /**
     * NEW in 4.1.2: our plugin no longer uses its own version of jquery; using Wordpress' jquery
     */
    public function addStylesheets()
    {
        wp_enqueue_style('jquery'); // use WP jquery
        self::addStylesheet('bootstrap');
        self::addStylesheet('plugin');
        self::addStylesheet('simple-tags');
    }

    /**
     * This method makes sure that all plugin versions still work.
     */
    public function grabOldConfig() 
    {
        $configPath = RIDDLE_PLUGIN_PATH . '/src/config/configs.php';
        if( !file_exists($configPath) ) { // The old config doesn't exist; no need to grab old data
            return;
        }

        $config = require $configPath;
        $config = json_decode($config, true);
        $pluginToken = $config['user']['wp_token'];

        UserSettings::saveKey($pluginToken); // save the grabbed key!
        unlink($configPath); // delete the old config!
    }

    /**
     * Init the session if it is not available already
     * The plugin uses the session to verify that the user has access to the leaderboard preview and no external user can access any of the previews
     */
    public static function initSession()
    {
        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
    }

    /**
     * This method gets triggered when the plugin gets activated
     */
    public function activate() 
    {
        UserSettings::addDefaults();
    }

    /**
     * This method gets triggered when the plugin gets deactivated
     */
    public function deactivate() 
    {
        UserSettings::destroySettings();
    }

    /**
     * This function loads all the scripts & stylesheets the plugin needs.
     * notice that these assets get only loaded if the user is on a riddle admin page.
     */
    public function loadAssets()
    {
        $this->addScripts();
        $this->addStylesheets();
    }

    private function _isOnRiddleAdminPage($pagePrefix = 'riddle')
    {
        $pageName = isset($_GET['page']) ? $_GET['page'] : false;

        if (!$pageName) {
            return false;
        }

        return substr($pageName, 0, strlen($pagePrefix)) === $pagePrefix;
    }

}