<?php

/**
 * Riddle custom result page manager / controller
 */

namespace src\controller;

use src\classes\Controller;
use src\classes\RiddleLoader;
use src\classes\UserSettings;
use src\classes\Landingpage\RiddleLandingpageManager;
use src\classes\Landingpage\RiddleLeaderboardPreviewHandler;
use src\classes\Landingpage\Type\LeaderboardType;

class RiddleCRPController extends Controller
{

    public static function riddleCustomResultPages()
    {
        global $riddleApiKey;
        
        if (!$riddleApiKey) {
            return self::view("error-nokey-defined");
        }

        if (!UserSettings::usesNewApi()) {
            return self::view('error-use-new-api');
        }

        $subpage = self::_getSubpage();

        if (!$subpage) { // no subpage is selected
            return self::view("riddle-custom-result-pages-list", [
                'pages' => RiddleLandingpageManager::getLandingpages(),
            ]);
        }

        $type = self::_getTypeFromSubpage($subpage);

        if (!$type) { // invalid subpage / type, redirect to index
            return self::redirectToAdminpage("riddle-custom-result-pages");
        }

        self::_insertParamsFromQuery($type, $subpage);
        
        $leaderboardHandler = new RiddleLeaderboardPreviewHandler(); // to make sure that only the logged in user has access to the preview
        $leaderboardHandler->addRiddleId($type->getValue('id'));

        return self::view('riddle-custom-result-pages-modify', [
            'availableFields' => self::_getAvailableFields($type),
            'fieldsView' => $type->getTypeFieldsView(),
            'type' => $type,
            'heading' => \ucfirst($subpage) . ' ' . $type->getType(),
            'submitText' => self::_getSubmitText($subpage),
            'previewUrl' => RIDDLE_URL_PATH . '/public/riddle-leaderboard-preview.php',
            'amountEntriesOptions' => [
                10 => 'Show 10 entries (default)',
                5 => 'Show x entries',
                -1 => 'Show all entries',
            ],
        ]);
    }

    private static function _getAvailableFields(LeaderboardType $type)
    {
        global $riddleLoader;

        $fieldsOrder = $type->getValue('leadfieldNamesOrder');
        $fieldsOrder = '' === $fieldsOrder ? [] : explode(',', $type->getValue('leadfieldNamesOrder')); // if previously saved - let's keep the order!

        $availableFields = [
            'index' => [
                'desc' => 'shows the ranking number, e.g. 1, 2, 3…',
            ],
            'percentage' => [
                'desc' => 'each user’s score, as %'
            ], 
            'latestScore' => [
                'desc' => 'the user\'s latest score e.g. 9/10 => 9',
            ], 
            'scoreSum' => [
                'desc' => 'the user\'s sum of all scores e.g. 9/10 & 8/10 => 9+8 = 17',
            ],
            'time' => [
                'desc' => 'How long it took each user to complete the quiz',
            ],
        ];
        $leadFields = $riddleLoader->getApi()->getLeadFields($type->getValue('id'));
        $leadFields = array_merge($availableFields, $leadFields);

        $fields = [];

        foreach ($fieldsOrder as $fieldName) { // put the fields back into the used order
            if (isset($leadFields[$fieldName])) {
                $fields[$fieldName] = $leadFields[$fieldName];
                unset($leadFields[$fieldName]);
            }
        }

        return array_merge(
            $fields,
            $leadFields // add the fields that are not included in the order
        );
    }

    public static function riddleHandleLandingpageOperation()
    {
        $subpage = self::_getSubpage();

        if (('create' !== $subpage && 'edit' !== $subpage) || !isset($_POST['riddle_type_sent'])) {
            return false;
        }
        
        $type = self::_getTypeFromQuery($subpage);

        if (!$type) {
            return false; // invalid type; just skip this action
        }

        $id = $type->update();
        RiddleLandingpageManager::save();

        return self::redirectToAdminpage('riddle-custom-result-pages&subpage=edit&id=' . $id); // back to the edit page
    }

    public static function riddleHandleLandingpageDeletion()
    {
        $subpage = self::_getSubpage();

        if ('delete' !== $subpage) {
            return false;
        }

        $type = self::_getTypeFromQuery($subpage);

        if (!$type) {
            return false;
        }

        $id = isset($_GET['id']) ? $_GET['id'] : -1;
        RiddleLandingpageManager::delete($id);

        return self::redirectToAdminpage('riddle-custom-result-pages'); // back to the CRP list
    }

    private static function _getSubmitText($subpage)
    {
        if ('create' === $subpage) {
            return 'CREATE';
        }

        if ('edit' === $subpage) {
            return 'SAVE CHANGES';
        }

        return 'EDIT';
    }

    private static function _insertParamsFromQuery(&$type, $subpage)
    {
        if (isset($_GET['riddleId'])) {
            $type->setValue('id', intval($_GET['riddleId']));
        }

        if (isset($_GET['slug'])) {
            $type->setValue('slug', urldecode($_GET['slug']));
        }
    }

    private static function _getTypeFromSubpage($subpage)
    {
        if ('create' === $subpage) {
            $typeName = self::_getGETValue('type');

            return RiddleLandingpageManager::getTypeInstance($typeName);
        }

        if ('edit' === $subpage) {
            $pageId = self::_getGETValue('id');
            
            return RiddleLandingpageManager::getTypeById($pageId);
        }

        return null;
    }

    private static function _getTypeFromQuery($subpage)
    {
        if (isset($_GET['type']) && 'create' === $subpage) {
            $typeName = self::_getGETValue('type');
            return RiddleLandingpageManager::getTypeInstance($typeName);
        }

        if (isset($_GET['id']) && ('edit' === $subpage || 'delete' === $subpage)) {
            return RiddleLandingpageManager::getTypeById($_GET['id']);
        }

        return null;
    }

}